"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FloatingWindow = void 0;
const React = require("react");
const react_dom_1 = require("react-dom");
const Rect_1 = require("../Rect");
const Types_1 = require("../Types");
/** @internal */
const FloatingWindow = (props) => {
    // Extraction des propriétés du composant
    const { title, id, url, rect, onCloseWindow, onSetWindow, children } = props;
    // Référence à la fenêtre flottante
    const popoutWindow = React.useRef(null);
    // Référence à un timer
    const timerId = React.useRef(null);
    // État pour stocker le contenu de la fenêtre flottante
    const [content, setContent] = React.useState(undefined);
    // Effet de layout qui s'exécute une fois au montage du composant
    React.useLayoutEffect(() => {
        // Nettoyer le timer s'il existe
        if (timerId.current) {
            clearTimeout(timerId.current);
        }
        // Variable pour vérifier si le composant est toujours monté
        let isMounted = true;
        // Calcul des dimensions de la fenêtre flottante
        const r = rect || new Rect_1.Rect(0, 0, 100, 100);
        // Récupération des styles de la fenêtre principale
        const styles = Array.from(window.document.styleSheets).reduce((result, styleSheet) => {
            let rules = undefined;
            try {
                rules = styleSheet.cssRules;
            }
            catch (e) {
                // styleSheet.cssRules peut lever une exception de sécurité
            }
            try {
                return [
                    ...result,
                    {
                        href: styleSheet.href,
                        type: styleSheet.type,
                        rules: rules ? Array.from(rules).map(rule => rule.cssText) : null,
                    }
                ];
            }
            catch (e) {
                return result;
            }
        }, []);
        // Ouverture de la fenêtre flottante
        popoutWindow.current = window.open(url, id, `left=${r.x},top=${r.y},width=${r.width},height=${r.height}`);
        if (popoutWindow.current !== null) {
            // Appel de la fonction onSetWindow avec l'ID et la fenêtre flottante
            onSetWindow(id, popoutWindow.current);
            // Écoute de l'événement de déchargement du parent pour fermer toutes les fenêtres flottantes
            window.addEventListener("beforeunload", () => {
                if (popoutWindow.current) {
                    popoutWindow.current.close();
                    popoutWindow.current = null;
                }
            });
            // Écoute de l'événement de chargement de la fenêtre flottante
            popoutWindow.current.addEventListener("load", () => {
                if (isMounted) {
                    const popoutDocument = popoutWindow.current.document;
                    // Définition du titre de la fenêtre flottante
                    popoutDocument.title = title;
                    // Création d'un élément div pour le contenu de la fenêtre flottante
                    const popoutContent = popoutDocument.createElement("div");
                    popoutContent.className = Types_1.CLASSES.FLEXLAYOUT__FLOATING_WINDOW_CONTENT;
                    // Ajout de l'élément div au corps de la fenêtre flottante
                    popoutDocument.body.appendChild(popoutContent);
                    // Copie des styles de la fenêtre principale vers la fenêtre flottante
                    copyStyles(popoutDocument, styles).then(() => {
                        setContent(popoutContent);
                    });
                    // Écoute de l'événement de déchargement de la fenêtre flottante
                    popoutWindow.current.addEventListener("beforeunload", () => {
                        onCloseWindow(id);
                    });
                }
            });
        }
        else {
            // Avertissement si l'ouverture de la fenêtre flottante échoue
            console.warn(`Unable to open window ${url}`);
            // Appel de la fonction onCloseWindow avec l'ID pour effectuer des actions de nettoyage
            onCloseWindow(id);
        }
        // Fonction de nettoyage qui s'exécute lorsque le composant est démonté
        return () => {
            isMounted = false;
            // Délai pour que le rafraîchissement ferme la fenêtre flottante
            timerId.current = setTimeout(() => {
                if (popoutWindow.current) {
                    popoutWindow.current.close();
                    popoutWindow.current = null;
                }
            }, 0);
        };
    }, []);
    // Rendu du contenu dans la fenêtre flottante à l'aide de createPortal
    if (content !== undefined) {
        return (0, react_dom_1.createPortal)(children, content);
    }
    else {
        return null;
    }
};
exports.FloatingWindow = FloatingWindow;
/** @internal */
function copyStyles(doc, styleSheets) {
    // Récupération de la balise head du document
    const head = doc.head;
    // Tableau de promesses pour suivre le chargement des styles
    const promises = [];
    // Copie des styles au niveau de la racine (html ou body)
    const rootStyles = window.getComputedStyle(document.documentElement).cssText;
    doc.documentElement.style.cssText = rootStyles;
    // Copie des styles définis dans les feuilles de style
    for (const styleSheet of styleSheets) {
        if (styleSheet.href) {
            // Préférez les liens car ils conservent les chemins vers les images, etc.
            const styleElement = doc.createElement("link");
            styleElement.type = styleSheet.type;
            styleElement.rel = "stylesheet";
            styleElement.href = styleSheet.href;
            head.appendChild(styleElement);
            // Création d'une promesse qui résout lorsque le lien de style est chargé
            promises.push(new Promise((resolve) => {
                styleElement.onload = () => resolve(true);
            }));
        }
        else {
            if (styleSheet.rules) {
                // Copie des styles directement si c'est une feuille de style inline
                const style = doc.createElement("style");
                for (const rule of styleSheet.rules) {
                    style.appendChild(doc.createTextNode(rule));
                }
                head.appendChild(style);
            }
        }
    }
    // Résolution de toutes les promesses pour indiquer que toutes les feuilles de style ont été chargées
    return Promise.all(promises);
}
//# sourceMappingURL=FloatingWindow.js.map